@@ -141,8 +141,8 @@ formatting = {
             'unfocused': 'roleName + pause + nestingLevel + pause + displayedText',
             },
         pyatspi.ROLE_CANVAS: {
-            'focused': 'labelAndName + imageDescription + roleName + pause + positionInList',
-            'unfocused': 'labelAndName + imageDescription + roleName + pause + positionInList',
+            'focused': 'labelAndName + (imageDescription or roleName) + pause + positionInList',
+            'unfocused': 'labelAndName + (imageDescription or roleName) + pause + positionInList',
             'basicWhereAmI': 'parentRoleName + pause + labelAndName + pause + selectedItemCount + pause',
             'detailedWhereAmI': 'parentRoleName + pause + labelAndName + pause + selectedItemCount + pause + selectedItems + pause'
             },
@@ -168,7 +168,7 @@ formatting = {
             'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
             },
         pyatspi.ROLE_COMBO_BOX: {
-            'focused': 'expandableState',
+            'focused': 'labelAndName + roleName + expandableState',
             'unfocused': 'labelAndName + roleName + pause + (currentLineText + anyTextSelection or positionInList) + ' + MNEMONIC + ' + accelerator',
             'basicWhereAmI': 'label + roleName + pause + name + (currentLineText + anyTextSelection or positionInList) + ' + MNEMONIC + ' + accelerator'
             },
@@ -176,13 +176,19 @@ formatting = {
             'focused': 'labelOrName + roleName',
             'unfocused': 'labelOrName + roleName + pause + currentLineText + allTextSelection',
             },
+        pyatspi.ROLE_DESCRIPTION_TERM: {
+            'unfocused': '(labelOrName or (displayedText + allTextSelection))',
+            },
+        pyatspi.ROLE_DESCRIPTION_VALUE: {
+            'unfocused': '(labelOrName or (displayedText + allTextSelection))',
+            },
         pyatspi.ROLE_DIAL: {
             'focused': 'value',
             'unfocused': 'labelOrName + roleName + value + required + availability + ' + MNEMONIC,
             'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
         pyatspi.ROLE_DIALOG: {
-            'focused': 'labelOrName + roleName + description',
+            'focused': 'labelOrName + roleName + (unrelatedLabels or description)',
             'unfocused': 'expandedEOCs or (labelOrName + (unrelatedLabels or description))'
             },
         pyatspi.ROLE_DOCUMENT_FRAME: {
@@ -237,8 +243,8 @@ formatting = {
             'detailedWhereAmI': 'label + readOnly + textRole + pause + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
         pyatspi.ROLE_ICON: {
-            'focused': 'labelAndName + imageDescription + roleName + pause + positionInList',
-            'unfocused': 'labelAndName + imageDescription + roleName + pause + positionInList',
+            'focused': 'labelAndName + (imageDescription or roleName) + pause + positionInList',
+            'unfocused': 'labelAndName + (imageDescription or roleName) + pause + positionInList',
             'basicWhereAmI': 'parentRoleName + pause + labelAndName + pause + selectedItemCount + pause',
             'detailedWhereAmI': 'parentRoleName + pause + labelAndName + pause + selectedItemCount + pause + selectedItems + pause'
             },
@@ -254,17 +260,17 @@ formatting = {
             'basicWhereAmI': 'label + displayedText + allTextSelection + roleName'
             },
         pyatspi.ROLE_LANDMARK: {
-            'focused': 'leaving or (labelAndName + roleName)',
-            'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + unrelatedLabels'
+            'focused': 'leaving or (roleName + labelAndName)',
+            'unfocused': 'roleName + (labelAndName or (substring and currentLineText)) + pause + unrelatedLabels'
             },
         pyatspi.ROLE_LAYERED_PANE: {
-            'focused': 'labelAndName + roleName + availability + noShowingChildren',
-            'unfocused': 'labelAndName + roleName + availability + noShowingChildren',
-            'basicWhereAmI': 'labelAndName + pause + roleName + pause + selectedItemCount + pause',
-            'detailedWhereAmI': 'labelAndName + pause + roleName + pause + selectedItemCount + pause + selectedItems + pause'
+            'focused': '(labelAndName or roleName) + availability + noShowingChildren',
+            'unfocused': '(labelAndName or roleName) + availability + noShowingChildren',
+            'basicWhereAmI': '(labelAndName or roleName) + pause + selectedItemCount + pause',
+            'detailedWhereAmI': '(labelAndName or roleName) + pause + roleName + pause + selectedItemCount + pause + selectedItems + pause'
             },
         pyatspi.ROLE_LINK: {
-            'unfocused': '(displayedText or name) + roleName + pause + expandableState + availability + ' + MNEMONIC,
+            'unfocused': '(name or displayedText) + roleName + pause + expandableState + availability + ' + MNEMONIC,
             'basicWhereAmI': 'linkInfo + pause + siteDescription + pause + fileSize + pause + ' + MNEMONIC
             },
         pyatspi.ROLE_LIST: {
@@ -342,7 +348,7 @@ formatting = {
             'basicWhereAmI': 'parentRoleName + pause + labelOrName + roleName + pause + positionInList + ' + MNEMONIC + ' + accelerator'
             },
         pyatspi.ROLE_PANEL: {
-            'focused': 'leaving or (labelAndName + roleName)',
+            'focused': 'leaving or (labelAndName + roleName + pause + unrelatedLabels)',
             'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + unrelatedLabels'
             },
         pyatspi.ROLE_PARAGRAPH: {
@@ -392,8 +398,8 @@ formatting = {
             'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
         pyatspi.ROLE_SECTION: {
-            'focused': 'labelOrName + currentLineText + allTextSelection + roleName',
-            'unfocused': 'labelOrName + currentLineText + allTextSelection + roleName + ' + MNEMONIC,
+            'focused': '(labelOrName or (currentLineText + allTextSelection)) + roleName',
+            'unfocused': '(labelOrName or (currentLineText + allTextSelection)) + roleName + ' + MNEMONIC,
             },
         pyatspi.ROLE_SLIDER: {
             'focused': 'value',
@@ -447,6 +453,8 @@ formatting = {
             },
         pyatspi.ROLE_TABLE_ROW: {
             'focused': 'expandableState',
+            'unfocused': '(labelOrName or displayedText) + pause + expandableState + pause + positionInList',
+            'basicWhereAmI': '(labelOrName or displayedText) + roleName + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel)'
             },
         pyatspi.ROLE_TEAROFF_MENU_ITEM: {
             'focused': '[]',
@@ -535,6 +543,10 @@ formatting = {
             'unfocused': '[Component(obj,\
                                      asString(label + displayedText + roleName + (description and space(": ") + description)))]',
             },
+        pyatspi.ROLE_APPLICATION: {
+            'focused':   '[Component(obj, asString(name + roleName))]',
+            'unfocused': '[Component(obj, asString(name + roleName))]',
+            },
         pyatspi.ROLE_ARTICLE: {
             'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
                           or ([Component(obj, asString(labelAndName + roleName))]\
@@ -580,6 +592,12 @@ formatting = {
                           + (readOnly and [Region(" " + asString(readOnly))])',
             'unfocused': BRAILLE_TEXT
             },
+        pyatspi.ROLE_DOCUMENT_WEB: {
+            'focused': '[Text(obj, asString(placeholderText), asString(eol), startOffset, endOffset)]\
+                          + (required and [Region(" " + asString(required))])\
+                          + (readOnly and [Region(" " + asString(readOnly))])',
+            'unfocused': BRAILLE_TEXT
+            },
         pyatspi.ROLE_ENTRY: {
             'unfocused': BRAILLE_TEXT
             },
@@ -589,9 +607,9 @@ formatting = {
             },
         pyatspi.ROLE_FRAME: {
             'focused':   '[Component(obj,\
-                                     asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]',
+                                     asString(labelOrName + roleName + alertAndDialogCount))]',
             'unfocused': '[Component(obj,\
-                                     asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]'
+                                     asString(labelOrName + roleName + alertAndDialogCount))]',
             },
         pyatspi.ROLE_HEADING: {
             'unfocused': '[Text(obj, asString(placeholderText), "", startOffset, endOffset)]\
