@@ -29,6 +29,7 @@ __copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
                 "Copyright (c) 2012 Igalia, S.L."
 __license__   = "LGPL"
 
+import faulthandler
 import gi
 import importlib
 import os
@@ -69,6 +70,7 @@ from . import event_manager
 from . import keybindings
 from . import logger
 from . import messages
+from . import mouse_review
 from . import notification_messages
 from . import orca_state
 from . import orca_platform
@@ -76,20 +78,14 @@ from . import script_manager
 from . import settings
 from . import settings_manager
 from . import speech
+from . import sound
 from .input_event import BrailleEvent
-from .input_event import KeyboardEvent
 
 _eventManager = event_manager.getManager()
 _scriptManager = script_manager.getManager()
 _settingsManager = settings_manager.getManager()
 _logger = logger.getLogger()
 
-try:
-    # If we don't have an active desktop, we will get a RuntimeError.
-    from . import mouse_review
-except RuntimeError:
-    pass
-
 def onEnabledChanged(gsetting, key):
     try:
         enabled = gsetting.get_boolean(key)
@@ -139,163 +135,53 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
     """
 
     if not force and obj == orca_state.locusOfFocus:
+        msg = "ORCA: Setting locusOfFocus to existing locusOfFocus"
+        debug.println(debug.LEVEL_INFO, msg, True)
         return
 
-    # If this event is not for the currently active script, then just return.
-    #
-    if event and event.source and \
-       event.host_application and orca_state.activeScript:
-        currentApp = orca_state.activeScript.app
-        try:
-            appList = [event.host_application, event.source.getApplication()]
-        except (LookupError, RuntimeError):
-            appList = []
-            debug.println(debug.LEVEL_SEVERE,
-                           "orca.setLocusOfFocus() application Error")
-        if not currentApp in appList:
+    if event and (orca_state.activeScript and not orca_state.activeScript.app):
+        script = _scriptManager.getScript(event.host_application, event.source)
+        _scriptManager.setActiveScript(script, "Setting locusOfFocus")
+
+    oldFocus = orca_state.locusOfFocus
+    try:
+        oldFocus.getRole()
+    except:
+        msg = "ORCA: Old locusOfFocus is null or defunct"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        oldFocus = None
+
+    if not obj:
+        msg = "ORCA: New locusOfFocus is null (being cleared)"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        orca_state.locusOfFocus = None
+        return
+
+    if orca_state.activeScript:
+        msg = "ORCA: Active script is: %s" % orca_state.activeScript
+        debug.println(debug.LEVEL_INFO, msg, True)
+        if orca_state.activeScript.utilities.isZombie(obj):
+            msg = "ERROR: New locusOfFocus (%s) is zombie. Not updating." % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+        if orca_state.activeScript.utilities.isDead(obj):
+            msg = "ERROR: New locusOfFocus (%s) is dead. Not updating." % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
             return
 
-    oldLocusOfFocus = orca_state.locusOfFocus
-    try:
-        # Just to see if we have a valid object.
-        oldLocusOfFocus.getRole()
-    except:
-        # Either it's None or it's an invalid remote object.
-        oldLocusOfFocus = None
-
+    msg = "ORCA: Changing locusOfFocus from %s to %s" % (oldFocus, obj)
+    debug.println(debug.LEVEL_INFO, msg, True)
     orca_state.locusOfFocus = obj
-    try:
-        app = orca_state.locusOfFocus.getApplication()
-    except:
-        orca_state.locusOfFocus = None
-        if event:
-            debug.println(debug.LEVEL_FINE,
-                          "LOCUS OF FOCUS: None event='%s'" % event.type)
-        else:
-            debug.println(debug.LEVEL_FINE,
-                          "LOCUS OF FOCUS: None event=None")
-    else:
-        try:
-            appname = "'" + app.name + "'"
-        except:
-            appname = "None"
-        try:
-            name = orca_state.locusOfFocus.name
-            rolename = orca_state.locusOfFocus.getRoleName()
-        except:
-            name = "Error"
-            rolename = "Error"
-        debug.println(debug.LEVEL_FINE,
-                      "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
-                      % (appname, name, rolename))
 
-        if event:
-            debug.println(debug.LEVEL_FINE,
-                          "                event='%s'" % event.type)
-        else:
-            debug.println(debug.LEVEL_FINE,
-                          "                event=None")
+    if not notifyScript:
+        return
 
-    if notifyScript and orca_state.activeScript:
-        orca_state.activeScript.locusOfFocusChanged(
-            event, oldLocusOfFocus, orca_state.locusOfFocus)
+    if not orca_state.activeScript:
+        msg = "ORCA: Cannot notify active script because there isn't one"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        return
 
-########################################################################
-#                                                                      #
-# METHODS FOR PRE-PROCESSING AND MASSAGING KEYBOARD EVENTS.            #
-#                                                                      #
-########################################################################
-
-_orcaModifierPressed = False
-
-def _processKeyboardEvent(event):
-    """The primary key event handler for Orca.  Keeps track of various
-    attributes, such as the lastInputEvent.  Also does key echo as well
-    as any local keybindings before passing the event on to the active
-    script.  This method is called synchronously from the AT-SPI registry
-    and should be performant.  In addition, it must return True if it has
-    consumed the event (and False if not).
-
-    Arguments:
-    - event: an AT-SPI DeviceEvent
-
-    Returns True if the event should be consumed.
-    """
-    global _orcaModifierPressed
-
-    keyboardEvent = KeyboardEvent(event)
-    debug.println(debug.LEVEL_FINE, keyboardEvent.toString())
-
-    # Weed out duplicate and otherwise bogus events.
-    # TODO - JD: Be sure these are the right values to return
-    if not keyboardEvent.timestamp:
-        debug.println(debug.LEVEL_FINE, "IGNORING EVENT: NO TIMESTAMP")
-        return False
-    if keyboardEvent == orca_state.lastInputEvent:
-        debug.println(debug.LEVEL_FINE, "IGNORING EVENT: DUPLICATE")
-        return False
-
-    # Figure out what we've got.
-    isOrcaModifier = keyboardEvent.isOrcaModifier()
-    isPressedEvent = keyboardEvent.isPressedKey()
-    if isOrcaModifier:
-        _orcaModifierPressed = isPressedEvent
-    if _orcaModifierPressed:
-        keyboardEvent.modifiers |= keybindings.ORCA_MODIFIER_MASK
-
-    # Update our state.
-    orca_state.lastInputEvent = keyboardEvent
-    if not keyboardEvent.isModifierKey():
-        keyboardEvent.setClickCount()
-        orca_state.lastNonModifierKeyEvent = keyboardEvent
-
-    # Echo it based on what it is and the user's settings.
-    script = orca_state.activeScript
-    if not script:
-        debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO NO SCRIPT")
-        return False
-
-    if isPressedEvent:
-        if not orca_state.activeWindow:
-            orca_state.activeWindow = script.utilities.activeWindow()
-        script.presentationInterrupt()
-    script.presentKeyboardEvent(keyboardEvent)
-    if keyboardEvent.isModifierKey() and not isOrcaModifier:
-        return False
- 
-    # Special modes.
-    if not isPressedEvent and keyboardEvent.event_string == "Escape":
-        script.exitLearnMode(keyboardEvent)
-    if orca_state.learnModeEnabled and not keyboardEvent.modifiers:
-        if keyboardEvent.event_string == "F1":
-            orca_state.learnModeEnabled = False
-            return helpForOrca()
-        if isPressedEvent and keyboardEvent.event_string in ["F2", "F3"]:
-            return script.listOrcaShortcuts(keyboardEvent)
-    if orca_state.capturingKeys:
-        return False
-    if notification_messages.listNotificationMessagesModeEnabled:
-        return notification_messages.listNotificationMessages(script, keyboardEvent)
-
-    # See if the event manager wants it (i.e. it is bound to a command.
-    if _eventManager.processKeyboardEvent(keyboardEvent):
-        return True
-
-    # Do any needed xmodmap crap.
-    global _restoreOrcaKeys
-    if not isPressedEvent:
-        if keyboardEvent.event_string in settings.orcaModifierKeys \
-           and orca_state.bypassNextCommand:
-            _restoreXmodmap()
-            _restoreOrcaKeys = True
-        elif _restoreOrcaKeys and not orca_state.bypassNextCommand:
-            _createOrcaXmodmap()
-            _restoreOrcaKeys = False
-    elif not keyboardEvent.isModifierKey():
-        _orcaModifierPressed = False
-        orca_state.bypassNextCommand = False
- 
-    return isOrcaModifier or orca_state.learnModeEnabled
+    orca_state.activeScript.locusOfFocusChanged(event, oldFocus, orca_state.locusOfFocus)
 
 ########################################################################
 #                                                                      #
@@ -337,6 +223,23 @@ def _processBrailleEvent(event):
 #                                                                      #
 ########################################################################
 
+def updateKeyMap(keyboardEvent):
+    """Unsupported convenience method to call sad hacks which should go away."""
+
+    global _restoreOrcaKeys
+    if keyboardEvent.isPressedKey():
+        return
+
+    if keyboardEvent.event_string in settings.orcaModifierKeys \
+       and orca_state.bypassNextCommand:
+        _restoreXmodmap()
+        _restoreOrcaKeys = True
+        return
+
+    if _restoreOrcaKeys and not orca_state.bypassNextCommand:
+        _createOrcaXmodmap()
+        _restoreOrcaKeys = False
+
 def _setXmodmap(xkbmap):
     """Set the keyboard map using xkbcomp."""
     p = subprocess.Popen(['xkbcomp', '-w0', '-', os.environ['DISPLAY']],
@@ -347,33 +250,51 @@ def _setCapsLockAsOrcaModifier(enable):
     """Enable or disable use of the caps lock key as an Orca modifier key."""
     interpretCapsLineProg = re.compile(
         r'^\s*interpret\s+Caps[_+]Lock[_+]AnyOfOrNone\s*\(all\)\s*{\s*$', re.I)
-    capsModLineProg = re.compile(
-        r'^\s*action\s*=\s*SetMods\s*\(\s*modifiers\s*=\s*Lock\s*,\s*clearLocks\s*\)\s*;\s*$', re.I)
     normalCapsLineProg = re.compile(
         r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Lock\s*\)\s*;\s*$', re.I)
+    interpretShiftLineProg = re.compile(
+        r'^\s*interpret\s+Shift[_+]Lock[_+]AnyOf\s*\(\s*Shift\s*\+\s*Lock\s*\)\s*{\s*$', re.I)
+    normalShiftLineProg = re.compile(
+        r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Shift\s*\)\s*;\s*$', re.I)
+    disabledModLineProg = re.compile(
+        r'^\s*action\s*=\s*NoAction\s*\(\s*\)\s*;\s*$', re.I)
     normalCapsLine = '        action= LockMods(modifiers=Lock);'
-    capsModLine =    '        action= SetMods(modifiers=Lock,clearLocks);'
+    normalShiftLine = '        action= LockMods(modifiers=Shift);'
+    disabledModLine = '        action= NoAction();'
     lines = _originalXmodmap.decode('UTF-8').split('\n')
     foundCapsInterpretSection = False
-    for i in range(len(lines)):
-        line = lines[i]
-        if not foundCapsInterpretSection:
+    foundShiftInterpretSection = False
+    modified = False
+    for i, line in enumerate(lines):
+        if not foundCapsInterpretSection and not foundShiftInterpretSection:
             if interpretCapsLineProg.match(line):
                 foundCapsInterpretSection = True
-        else:
+            elif interpretShiftLineProg.match(line):
+                foundShiftInterpretSection = True
+        elif foundCapsInterpretSection:
             if enable:
                 if normalCapsLineProg.match(line):
-                    lines[i] = capsModLine
-                    _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
-                    return
+                    lines[i] = disabledModLine
+                    modified = True
             else:
-                if capsModLineProg.match(line):
+                if disabledModLineProg.match(line):
                     lines[i] = normalCapsLine
-                    _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
-                    return
+                    modified = True
             if line.find('}'):
-                # Failed to find the line we need to change
-                return
+                foundCapsInterpretSection = False
+        else: # foundShiftInterpretSection
+            if enable:
+                if normalShiftLineProg.match(line):
+                    lines[i] = disabledModLine
+                    modified = True
+            else:
+                if disabledModLineProg.match(line):
+                    lines[i] = normalShiftLine
+                    modified = True
+            if line.find('}'):
+                foundShiftInterpretSection = False
+    if modified:
+        _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
 
 def _createOrcaXmodmap():
     """Makes an Orca-specific Xmodmap so that the keys behave as we
@@ -383,7 +304,8 @@ def _createOrcaXmodmap():
     global _capsLockCleared
 
     cmd = []
-    if "Caps_Lock" in settings.orcaModifierKeys:
+    if "Caps_Lock" in settings.orcaModifierKeys \
+       or "Shift_Lock" in settings.orcaModifierKeys:
         _setCapsLockAsOrcaModifier(True)
         _capsLockCleared = True
     elif _capsLockCleared:
@@ -421,12 +343,14 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     Returns True to indicate the input event has been consumed.
     """
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Loading User Settings')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Loading User Settings', True)
 
     global _userSettings
 
     # Shutdown the output drivers and give them a chance to die.
 
+    player = sound.getPlayer()
+    player.shutdown()
     speech.shutdown()
     braille.shutdown()
 
@@ -440,7 +364,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
             _settingsManager.setProfile(_profile)
             reloaded = True
         except ImportError:
-            debug.printException(debug.LEVEL_FINEST)
+            debug.printException(debug.LEVEL_INFO)
         except:
             debug.printException(debug.LEVEL_SEVERE)
     else:
@@ -448,42 +372,41 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         try:
             _userSettings = _settingsManager.getGeneralSettings(_profile)
         except ImportError:
-            debug.printException(debug.LEVEL_FINEST)
+            debug.printException(debug.LEVEL_INFO)
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
+    if not script:
+        script = _scriptManager.getDefaultScript()
+
     _settingsManager.loadAppSettings(script)
 
     if _settingsManager.getSetting('enableSpeech'):
         try:
             speech.init()
             if reloaded and not skipReloadMessage:
-                speech.speak(messages.SETTINGS_RELOADED,
-                             settings.voices.get(settings.SYSTEM_VOICE))
-            debug.println(debug.LEVEL_CONFIGURATION,
-                          "Speech module has been initialized.")
+                script.speakMessage(messages.SETTINGS_RELOADED)
         except:
             debug.printException(debug.LEVEL_SEVERE)
-            debug.println(debug.LEVEL_SEVERE,
-                          "Could not initialize connection to speech.")
     else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Speech module has NOT been initialized.")
+        msg = 'ORCA: Speech is not enabled in settings'
+        debug.println(debug.LEVEL_INFO, msg, True)
 
     if _settingsManager.getSetting('enableBraille'):
         try:
             braille.init(_processBrailleEvent, settings.tty)
         except:
             debug.printException(debug.LEVEL_WARNING)
-            debug.println(debug.LEVEL_WARNING,
-                          "Could not initialize connection to braille.")
+            msg = 'ORCA: Could not initialize connection to braille.'
+            debug.println(debug.LEVEL_WARNING, msg, True)
 
-    # I'm not sure where else this should go. But it doesn't really look
-    # right here.
-    try:
-        mouse_review.mouse_reviewer.toggle(on=settings.enableMouseReview)
-    except NameError:
-        pass
+    if _settingsManager.getSetting('enableMouseReview'):
+        mouse_review.reviewer.activate()
+    else:
+        mouse_review.reviewer.deactivate()
+
+    if _settingsManager.getSetting('enableSound'):
+        player.init()
 
     global _orcaModifiers
     custom = [k for k in settings.orcaModifierKeys if k not in _orcaModifiers]
@@ -499,7 +422,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     _scriptManager.activate()
     _eventManager.activate()
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: User Settings Loaded')
+    debug.println(debug.LEVEL_INFO, 'ORCA: User Settings Loaded', True)
 
     return True
 
@@ -524,7 +447,7 @@ def _showPreferencesUI(script, prefs):
     orca_state.orcaOS.showGUI()
 
 def showAppPreferencesGUI(script=None, inputEvent=None):
-    """Displays the user interace to configure the settings for a
+    """Displays the user interface to configure the settings for a
     specific applications within Orca and set up those app-specific
     user preferences using a GUI.
 
@@ -541,7 +464,7 @@ def showAppPreferencesGUI(script=None, inputEvent=None):
     return True
 
 def showPreferencesGUI(script=None, inputEvent=None):
-    """Displays the user interace to configure Orca and set up
+    """Displays the user interface to configure Orca and set up
     user preferences using a GUI.
 
     Returns True to indicate the input event has been consumed.
@@ -558,6 +481,7 @@ def helpForOrca(script=None, inputEvent=None, page=""):
 
     Returns True to indicate the input event has been consumed.
     """
+    orca_state.learnModeEnabled = False
     uri = "help:orca"
     if page:
         uri += "?%s" % page
@@ -578,7 +502,7 @@ def quitOrca(script=None, inputEvent=None):
     return True
 
 def showFindGUI(script=None, inputEvent=None):
-    """Displays the user interace to perform an Orca Find.
+    """Displays the user interface to perform an Orca Find.
 
     Returns True to indicate the input event has been consumed.
     """
@@ -602,7 +526,7 @@ def init(registry):
     module has already been initialized.
     """
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Initializing Orca module')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Initializing', True)
 
     global _initialized
 
@@ -616,7 +540,6 @@ def init(registry):
         signal.alarm(settings.timeoutTime)
 
     loadUserSettings()
-    _eventManager.registerKeystrokeListener(_processKeyboardEvent)
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
@@ -628,7 +551,7 @@ def init(registry):
     if a11yAppSettings:
         a11yAppSettings.connect('changed', onEnabledChanged)
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Orca module initialized')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Initialized', True)
 
     return True
 
@@ -636,7 +559,7 @@ def start(registry, cacheValues):
     """Starts Orca.
     """
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Starting Orca')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Starting', True)
 
     if not _initialized:
         init(registry)
@@ -653,7 +576,11 @@ def start(registry, cacheValues):
     if cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Orca starting registry')
+    Gdk.notify_startup_complete()
+    msg = 'ORCA: Startup complete notification made'
+    debug.println(debug.LEVEL_INFO, msg, True)
+
+    debug.println(debug.LEVEL_INFO, 'ORCA: Starting registry', True)
     registry.start(gil=False)
 
 def die(exitCode=1):
@@ -669,8 +596,8 @@ def die(exitCode=1):
         os.kill(pid, signal.SIGTERM)
 
 def timeout(signum=None, frame=None):
-    debug.println(debug.LEVEL_SEVERE,
-                  "TIMEOUT: something has hung.  Aborting.")
+    msg = 'TIMEOUT: something has hung. Aborting.'
+    debug.println(debug.LEVEL_SEVERE, msg, True)
     debug.printStack(debug.LEVEL_ALL)
     debug.examineProcesses()
     die(EXIT_CODE_HANG)
@@ -682,7 +609,7 @@ def shutdown(script=None, inputEvent=None):
     was never initialized.
     """
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Shutting down Orca')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Shutting down', True)
 
     global _initialized
 
@@ -698,8 +625,6 @@ def shutdown(script=None, inputEvent=None):
     orca_state.activeScript.presentMessage(messages.STOP_ORCA)
 
     _scriptManager.deactivate()
-
-    _eventManager.deregisterKeystrokeListener(_processKeyboardEvent)
     _eventManager.deactivate()
 
     # Shutdown all the other support.
@@ -708,6 +633,9 @@ def shutdown(script=None, inputEvent=None):
         speech.shutdown()
     if settings.enableBraille:
         braille.shutdown()
+    if settings.enableSound:
+        player = sound.getPlayer()
+        player.shutdown()
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
@@ -715,10 +643,10 @@ def shutdown(script=None, inputEvent=None):
     _initialized = False
     _restoreXmodmap(_orcaModifiers)
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Orca stopping registry')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Stopping registry', True)
     pyatspi.Registry.stop()
 
-    debug.println(debug.LEVEL_FINEST, 'INFO: Orca shutdown complete')
+    debug.println(debug.LEVEL_INFO, 'ORCA: Shutdown complete', True)
 
     return True
 
@@ -726,12 +654,8 @@ exitCount = 0
 def shutdownOnSignal(signum, frame):
     global exitCount
 
-    debug.println(debug.LEVEL_ALL,
-                  "Shutting down and exiting due to signal = %d" \
-                  % signum)
-
-    debug.println(debug.LEVEL_ALL, "Current stack is:")
-    debug.printStack(debug.LEVEL_ALL)
+    msg = 'ORCA: Shutting down and exiting due to signal=%d' % signum
+    debug.println(debug.LEVEL_INFO, msg, True)
 
     # Well...we'll try to exit nicely, but if we keep getting called,
     # something bad is happening, so just quit.
@@ -766,11 +690,10 @@ def shutdownOnSignal(signum, frame):
     if not cleanExit:
         die(EXIT_CODE_HANG)
 
-def abortOnSignal(signum, frame):
-    debug.println(debug.LEVEL_ALL,
-                  "Aborting due to signal = %d" \
-                  % signum)
-    die(signum)
+def crashOnSignal(signum, frame):
+    signal.signal(signum, signal.SIG_DFL)
+    _restoreXmodmap(_orcaModifiers)
+    os.kill(os.getpid(), signum)
 
 def main(cacheValues=True):
     """The main entry point for Orca.  The exit codes for Orca will
@@ -779,6 +702,11 @@ def main(cacheValues=True):
     an exit code of 0 means normal completion and an exit code of 50
     means Orca exited because of a hang."""
 
+    if debug.debugFile and os.path.exists(debug.debugFile.name):
+        faulthandler.enable(file=debug.debugFile, all_threads=False)
+    else:
+        faulthandler.enable(all_threads=False)
+
     # Method to call when we think something might be hung.
     #
     settings.timeoutCallback = timeout
@@ -789,7 +717,7 @@ def main(cacheValues=True):
     signal.signal(signal.SIGINT, shutdownOnSignal)
     signal.signal(signal.SIGTERM, shutdownOnSignal)
     signal.signal(signal.SIGQUIT, shutdownOnSignal)
-    signal.signal(signal.SIGSEGV, abortOnSignal)
+    signal.signal(signal.SIGSEGV, crashOnSignal)
 
     if not _settingsManager.isAccessibilityEnabled():
         _settingsManager.setAccessibility(True)
@@ -798,11 +726,8 @@ def main(cacheValues=True):
 
     try:
         message = messages.START_ORCA
-        if not _settingsManager.getSetting('onlySpeakDisplayedText'):
-            speech.speak(message, settings.voices.get(settings.SYSTEM_VOICE))
-        if _settingsManager.getSetting('enableBraille') \
-           or _settingsManager.getSetting('enableBrailleMonitor'):
-            braille.displayMessage(message)
+        script = _scriptManager.getDefaultScript()
+        script.presentMessage(message)
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
@@ -810,7 +735,18 @@ def main(cacheValues=True):
     if script:
         window = script.utilities.activeWindow()
         if window and not orca_state.locusOfFocus:
+            try:
+                app = window.getApplication()
+            except:
+                msg = "ORCA: Exception getting app for %s" % window
+                debug.println(debug.LEVEL_INFO, msg, True)
+            else:
+                script = _scriptManager.getScript(app, window)
+                _scriptManager.setActiveScript(script, "Launching.")
             setLocusOfFocus(None, window)
+            focusedObject = script.utilities.focusedObject(window)
+            if focusedObject:
+                setLocusOfFocus(None, focusedObject)
 
     try:
         start(pyatspi.Registry, cacheValues) # waits until we stop the registry
