@@ -37,6 +37,7 @@ class ScriptManager:
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Initializing', True)
         self.appScripts = {}
         self.toolkitScripts = {}
+        self.customScripts = {}
         self._appModules = apps.__all__
         self._toolkitModules = toolkits.__all__
         self._defaultScript = None
@@ -46,29 +47,21 @@ class ScriptManager:
              "orca.scripts.apps",
              "orca.scripts.toolkits"]
         self._appNames = \
-            {'Bon Echo':         'Mozilla',
-             'Deer Park':        'Mozilla',
-             'Firefox':          'Mozilla',
-             'Minefield':        'Mozilla',
-             'Namoroka':         'Mozilla',
-             'Shiretoko':        'Mozilla',
-             'Lanikai':          'Thunderbird',
-             'Mail/News':        'Thunderbird',
-             'Miramar':          'Thunderbird',
-             'Shredder':         'Thunderbird',
-             'Earlybird':        'Thunderbird',
-             'gaim':             'pidgin',
-             'empathy-chat':     'empathy',
+            {'Firefox': 'Mozilla',
+             'Icedove': 'Thunderbird',
+             'Nereid': 'Banshee',
+             'empathy-chat': 'empathy',
              'gnome-calculator': 'gcalctool',
-             'marco':            'metacity',
-             'Nereid':           'Banshee',
-             'vte':              'gnome-terminal',
-             'gnome-terminal-server': 'gnome-terminal',
+             'gtk-window-decorator': 'switcher',
+             'marco': 'switcher',
              'mate-notification-daemon': 'notification-daemon',
+             'metacity': 'switcher',
+             'pluma': 'gedit',
             }
 
         self.setActiveScript(None, "__init__")
         self._desktop = pyatspi.Registry.getDesktop(0)
+        self._active = False
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Initialized', True)
 
     def activate(self):
@@ -78,6 +71,7 @@ class ScriptManager:
         self._defaultScript = self.getScript(None)
         self._defaultScript.registerEventListeners()
         self.setActiveScript(self._defaultScript, "activate")
+        self._active = True
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Activated', True)
 
     def deactivate(self):
@@ -90,13 +84,15 @@ class ScriptManager:
         self.setActiveScript(None, "deactivate")
         self.appScripts = {}
         self.toolkitScripts = {}
+        self.customScripts = {}
+        self._active = False
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Deactivated', True)
 
     def getModuleName(self, app):
         """Returns the module name of the script to use for application app."""
 
         try:
-            appAndNameExist = app != None and app.name != ''
+            appAndNameExist = app is not None and app.name != ''
         except (LookupError, RuntimeError):
             appAndNameExist = False
             msg = 'ERROR: %s no longer exists' % app
@@ -105,10 +101,12 @@ class ScriptManager:
         if not appAndNameExist:
             return None
 
-        # Many python apps have an accessible name which ends in '.py'.
-        # Sometimes OOo has 'soffice.bin' as its name.
-        name = app.name.split('.')[0]
+        name = app.name
         altNames = list(self._appNames.keys())
+        if name.endswith(".py") or name.endswith(".bin"):
+            name = name.split('.')[0]
+        elif name.startswith("org.") or name.startswith("com."):
+            name = name.split('.')[-1]
 
         names = [n for n in altNames if n.lower() == name.lower()]
         if names:
@@ -139,6 +137,17 @@ class ScriptManager:
 
         return name
 
+    def _scriptForRole(self, obj):
+        try:
+            role = obj.getRole()
+        except:
+            return ''
+
+        if role == pyatspi.ROLE_TERMINAL:
+            return 'terminal'
+
+        return ''
+
     def _newNamedScript(self, app, name):
         """Attempts to locate and load the named module. If successful, returns
         a script based on this module."""
@@ -211,7 +220,32 @@ class ScriptManager:
 
         return script
 
-    def getScript(self, app, obj=None):
+    def sanityCheckScript(self, script):
+        if not self._active:
+            return script
+
+        try:
+            appInDesktop = script.app in self._desktop
+        except:
+            appInDesktop = False
+
+        if appInDesktop:
+            return script
+
+        msg = "WARNING: %s is not in the registry's desktop" % script.app
+        debug.println(debug.LEVEL_INFO, msg, True)
+
+        newScript = self._getScriptForAppReplicant(script.app)
+        if newScript:
+            msg = "SCRIPT MANAGER: Script for app replicant found: %s" % newScript
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return newScript
+
+        msg = "WARNING: Failed to get a replacement script for %s" % script.app
+        debug.println(debug.LEVEL_INFO, msg, True)
+        return script
+
+    def getScript(self, app, obj=None, sanityCheck=False):
         """Get a script for an app (and make it if necessary).  This is used
         instead of a simple calls to Script's constructor.
 
@@ -221,9 +255,19 @@ class ScriptManager:
         Returns an instance of a Script.
         """
 
+        customScript = None
         appScript = None
         toolkitScript = None
 
+        roleName = self._scriptForRole(obj)
+        if roleName:
+            customScripts = self.customScripts.get(app, {})
+            customScript = customScripts.get(roleName)
+            if not customScript:
+                customScript = self._newNamedScript(app, roleName)
+                customScripts[roleName] = customScript
+            self.customScripts[app] = customScripts
+
         objToolkit = self._toolkitForObject(obj)
         if objToolkit:
             toolkitScripts = self.toolkitScripts.get(app, {})
@@ -247,12 +291,25 @@ class ScriptManager:
             debug.println(debug.LEVEL_WARNING, msg, True)
             appScript = self.getDefaultScript()
 
+        if customScript:
+            return customScript
+
+        try:
+            role = obj.getRole()
+        except:
+            forceAppScript = False
+        else:
+            forceAppScript = role in [pyatspi.ROLE_FRAME, pyatspi.ROLE_STATUS_BAR]
+
         # Only defer to the toolkit script for this object if the app script
         # is based on a different toolkit.
-        if toolkitScript \
+        if toolkitScript and not forceAppScript \
            and not issubclass(appScript.__class__, toolkitScript.__class__):
             return toolkitScript
 
+        if app and sanityCheck:
+            appScript = self.sanityCheckScript(appScript)
+
         return appScript
 
     def setActiveScript(self, newScript, reason=None):
@@ -277,6 +334,27 @@ class ScriptManager:
               (newScript.name, reason)
         debug.println(debug.LEVEL_INFO, msg, True)
 
+    def _getScriptForAppReplicant(self, app):
+        if not self._active:
+            return None
+
+        def _pid(app):
+            try:
+                return app.get_process_id()
+            except:
+                return -1
+
+        pid = _pid(app)
+        if pid == -1:
+            return None
+
+        items = self.appScripts.items()
+        for a, script in items:
+            if a != app and _pid(a) == pid and a in self._desktop:
+                return script
+
+        return None
+
     def reclaimScripts(self):
         """Compares the list of known scripts to the list of known apps,
         deleting any scripts as necessary.
@@ -284,13 +362,27 @@ class ScriptManager:
 
         appList = list(self.appScripts.keys())
         try:
-            appList = [a for a in appList if a != None and a not in self._desktop]
+            appList = [a for a in appList if a is not None and a not in self._desktop]
         except:
             debug.printException(debug.LEVEL_FINEST)
             return
 
         for app in appList:
+            msg = "SCRIPT MANAGER: %s is no longer in registry's desktop" % app
+            debug.println(debug.LEVEL_INFO, msg, True)
+
             appScript = self.appScripts.pop(app)
+            newScript = self._getScriptForAppReplicant(app)
+            if newScript:
+                msg = "SCRIPT MANAGER: Script for app replicant found: %s" % newScript
+                debug.println(debug.LEVEL_INFO, msg, True)
+
+                attrs = appScript.getTransferableAttributes()
+                for attr, value in attrs.items():
+                    msg = "SCRIPT MANAGER: Setting %s to %s" % (attr, value)
+                    debug.println(debug.LEVEL_INFO, msg, True)
+                    setattr(newScript, attr, value)
+
             del appScript
 
             try:
@@ -298,9 +390,17 @@ class ScriptManager:
             except KeyError:
                 pass
             else:
-                for toolkitScript in list(toolkitScripts.values()):
+                for toolkitScript in toolkitScripts.values():
                     del toolkitScript
 
+            try:
+                customScripts = self.customScripts.pop(app)
+            except KeyError:
+                pass
+            else:
+                for customScript in customScripts.values():
+                    del customScript
+
             del app
 
 _manager = ScriptManager()
