@@ -73,11 +73,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                                pyatspi.ROLE_MENU_BAR,
                                pyatspi.ROLE_TOOL_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
+                             pyatspi.ROLE_HEADING,
                              pyatspi.ROLE_LABEL,
                              pyatspi.ROLE_LINK,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TEXT]
 
+        if self._script.utilities.isEditableDescendantOfComboBox(obj):
+            args['skipRoles'].append(pyatspi.ROLE_COMBO_BOX)
+
         result.extend(super()._generateAncestors(obj, **args))
 
         return result
@@ -116,6 +120,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return []
 
     def _generateDescription(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateDescription(obj, **args)
+
         if self._script.utilities.isZombie(obj):
             return []
 
@@ -126,7 +133,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if obj != orca_state.locusOfFocus:
             if role in [pyatspi.ROLE_ALERT, pyatspi.ROLE_DIALOG]:
                 return super()._generateDescription(obj, **args)
-            return []
+            if not args.get('inMouseReview'):
+                return []
 
         formatType = args.get('formatType')
         if formatType == 'basicWhereAmI' and self._script.utilities.isLiveRegion(obj):
@@ -157,31 +165,46 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return []
 
     def _generateLabelOrName(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateLabelOrName(obj, **args)
+
         if self._script.utilities.isTextBlockElement(obj) \
            and not self._script.utilities.isLandmark(obj) \
            and not self._script.utilities.isDPub(obj):
             return []
 
-        if self._script.utilities.inDocumentContent(obj) and obj.name:
-            result = [obj.name]
+        if obj.name:
+            name = obj.name
+            if not self._script.utilities.hasExplicitName(obj):
+                name = name.strip()
+            result = [name]
             result.extend(self.voice(speech_generator.DEFAULT))
             return result
 
         return super()._generateLabelOrName(obj, **args)
 
     def _generateName(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateName(obj, **args)
+
         if self._script.utilities.isTextBlockElement(obj) \
            and not self._script.utilities.isLandmark(obj) \
            and not self._script.utilities.isDPub(obj):
             return []
 
+        if obj.name and not self._script.utilities.hasValidName(obj):
+            return []
+
+        role = args.get('role', obj.getRole())
+        alwaysPresent = [pyatspi.ROLE_PUSH_BUTTON,
+                         pyatspi.ROLE_IMAGE]
+
         if obj.parent and obj.name and obj.name == obj.parent.name \
-           and obj != orca_state.locusOfFocus:
+           and obj != orca_state.locusOfFocus and role not in alwaysPresent:
             return []
 
         # TODO - JD: Once the formatting strings are vastly cleaned up
         # or simply removed, hacks like this won't be needed.
-        role = args.get('role', obj.getRole())
         if role in [pyatspi.ROLE_COMBO_BOX, pyatspi.ROLE_SPIN_BUTTON]:
             return super()._generateName(obj, **args)
 
@@ -189,17 +212,24 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
            and not self._script.utilities.hasExplicitName(obj):
             return []
 
-        if self._script.utilities.inDocumentContent(obj) and obj.name:
+        if obj.name:
             if self._script.utilities.preferDescriptionOverName(obj):
                 result = [obj.description]
             else:
-                result = [obj.name]
+                name = obj.name
+                if not self._script.utilities.hasExplicitName(obj):
+                    name = name.strip()
+                result = [name]
+
             result.extend(self.voice(speech_generator.DEFAULT))
             return result
 
         return super()._generateName(obj, **args)
 
     def _generateLabel(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateLabel(obj, **args)
+
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
@@ -261,9 +291,6 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
     # TODO - JD: Yet another dumb generator method we should kill.
     def _generateTextRole(self, obj, **args):
-        if self._script.inSayAll():
-            return []
-
         return self._generateRoleName(obj, **args)
 
     def getLocalizedRoleName(self, obj, **args):
@@ -308,6 +335,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         if not force:
             doNotSpeak = [pyatspi.ROLE_FOOTER,
+                          pyatspi.ROLE_FORM,
                           pyatspi.ROLE_LABEL,
                           pyatspi.ROLE_MENU_ITEM,
                           pyatspi.ROLE_PARAGRAPH,
@@ -332,6 +360,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 doNotSpeak.append(obj.getRole())
             if total > 1:
                 doNotSpeak.append(pyatspi.ROLE_ROW_HEADER)
+            if self._script.utilities.isMenuInCollapsedSelectElement(obj):
+                doNotSpeak.append(pyatspi.ROLE_MENU)
 
         if obj.getState().contains(pyatspi.STATE_EDITABLE):
             lastKey, mods = self._script.utilities.lastKeyAndModifiers()
@@ -375,6 +405,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     result.append(self.getLocalizedRoleName(obj, **args))
                     result.extend(acss)
 
+        elif role == pyatspi.ROLE_COMMENT:
+            if index == 0:
+                result.append(self.getLocalizedRoleName(obj, **args))
+                result.extend(acss)
+
         elif role not in doNotSpeak and args.get('priorObj') != obj:
             result.append(self.getLocalizedRoleName(obj, **args))
             result.extend(acss)
@@ -383,12 +418,10 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return result
 
         ancestorRoles = [pyatspi.ROLE_HEADING, pyatspi.ROLE_LINK]
-        if index == total - 1 \
-           and (role == pyatspi.ROLE_IMAGE or self._script.utilities.queryNonEmptyText(obj)):
-            speakRoles = lambda x: x and x.getRole() in ancestorRoles
-            ancestor = pyatspi.findAncestor(obj, speakRoles)
-            if ancestor and ancestor.getRole() != role:
-                result.extend(self._generateRoleName(ancestor))
+        speakRoles = lambda x: x and x.getRole() in ancestorRoles
+        ancestor = pyatspi.findAncestor(obj, speakRoles)
+        if ancestor and ancestor.getRole() != role and (index == total - 1 or obj.name == ancestor.name):
+            result.extend(self._generateRoleName(ancestor))
 
         return result
 
@@ -461,10 +494,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
            and not _settingsManager.getSetting('enablePositionSpeaking'):
             return []
 
-        # TODO - JD: We cannot do this for XUL (or whatever Firefox is
-        # using in its non-webcontent dialogs)
-        #if not self._script.utilities.inDocumentContent(obj):
-        #    return super()._generatePositionInList(obj, **args)
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generatePositionInList(obj, **args)
+
         menuRoles = [pyatspi.ROLE_MENU_ITEM,
                      pyatspi.ROLE_TEAROFF_MENU_ITEM,
                      pyatspi.ROLE_CHECK_MENU_ITEM,
@@ -495,6 +527,28 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         result.extend(self.voice(speech_generator.SYSTEM))
         return result
 
+    def _generateRealTableCell(self, obj, **args):
+        result = super()._generateRealTableCell(obj, **args)
+        if not self._script.inFocusMode():
+            return result
+
+        if _settingsManager.getSetting('speakCellCoordinates'):
+            label = self._script.utilities.labelForCellCoordinates(obj)
+            if label:
+                result.append(label)
+                result.extend(self.voice(speech_generator.SYSTEM))
+                return result
+
+            row, col = self._script.utilities.coordinatesForCell(obj)
+            if self._script.utilities.cellRowChanged(obj):
+                result.append(messages.TABLE_ROW % (row + 1))
+                result.extend(self.voice(speech_generator.SYSTEM))
+            if self._script.utilities.cellColumnChanged(obj):
+                result.append(messages.TABLE_COLUMN % (col + 1))
+                result.extend(self.voice(speech_generator.SYSTEM))
+
+        return result
+
     def _generateTableCellRow(self, obj, **args):
         if not self._script.inFocusMode():
             return super()._generateTableCellRow(obj, **args)
@@ -504,7 +558,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         isRow = lambda x: x and x.getRole() == pyatspi.ROLE_TABLE_ROW
         row = pyatspi.findAncestor(obj, isRow)
-        if row and row.name:
+        if row and row.name and not self._script.utilities.isLayoutOnly(row):
             return self.generate(row)
 
         return super()._generateTableCellRow(obj, **args)
@@ -550,7 +604,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return []
 
         result = []
-        contents = self._script.utilities.filterContentsForPresentation(contents, False)
+        contents = self._script.utilities.filterContentsForPresentation(contents, True)
         msg = "WEB: Generating speech contents (length: %i)" % len(contents)
         debug.println(debug.LEVEL_INFO, msg, True)
         for i, content in enumerate(contents):
