@@ -149,7 +149,7 @@ GENERAL_KEYBOARD_LAYOUT_DESKTOP = 1
 GENERAL_KEYBOARD_LAYOUT_LAPTOP  = 2
 
 DESKTOP_MODIFIER_KEYS = ["Insert", "KP_Insert"]
-LAPTOP_MODIFIER_KEYS  = ["Caps_Lock"]
+LAPTOP_MODIFIER_KEYS  = ["Caps_Lock", "Shift_Lock"]
 
 VERBOSITY_LEVEL_BRIEF   = 0
 VERBOSITY_LEVEL_VERBOSE = 1
